@@ -417,14 +417,16 @@ void cb_SortByFreq (void) {
 void DupCheck()
 {
 	Fl_Color call_clr = FL_BACKGROUND2_COLOR;
-//	if (progStatus.xml_logbook)
-//		if (xml_check_dup())
-//			call_clr = fl_rgb_color(
-//				progdefaults.dup_color.R,
-//				progdefaults.dup_color.G,
-//				progdefaults.dup_color.B);
-//	if (!progStatus.xml_logbook && qsodb.duplicate(
-	if (qsodb.duplicate(
+
+	if (progdefaults.xml_logbook)
+		if (xml_check_dup())
+			call_clr = fl_rgb_color(
+				progdefaults.dup_color.R,
+				progdefaults.dup_color.G,
+				progdefaults.dup_color.B);
+
+	if (!progdefaults.xml_logbook && qsodb.duplicate(
+//	if (qsodb.duplicate(
 			inpCall->value(),
 			zdate(), ztime(), progdefaults.timespan, progdefaults.duptimespan,
 			inpFreq->value(), progdefaults.dupband,
@@ -462,10 +464,12 @@ void SearchLastQSO(const char *callsign)
 	size_t len = strlen(callsign);
 	if (len < 3)
 		return;
-//	if (progStatus.xml_logbook) {
-//		if(xml_get_record(callsign))
-//			return;
-//	}
+
+	if (progdefaults.xml_logbook) {
+		if(xml_get_record(callsign))
+			return;
+	}
+
 	Fl::focus(inpCall);
 
 	char* re = new char[len + 3];
@@ -569,6 +573,7 @@ void clearRecord() {
 	inpIOTA_log->value("");
 	inpDXCC_log->value("");
 	inpCONT_log->value("");
+	inpCNTY_log->value("");
 	inpCQZ_log->value("");
 	inpITUZ_log->value("");
 	inpTX_pwr_log->value("");
@@ -611,6 +616,7 @@ void saveRecord() {
 	} else {
 		rec.putField(MYXCHG, inpMyXchg_log->value());
 	}
+	rec.putField(CNTY, inpCNTY_log->value());
 	rec.putField(IOTA, inpIOTA_log->value());
 	rec.putField(DXCC, inpDXCC_log->value());
 	rec.putField(CONT, inpCONT_log->value());
@@ -657,6 +663,7 @@ cQsoRec rec;
 	rec.putField(STX, inpSerNoOut_log->value());
 	rec.putField(XCHG1, inpXchgIn_log->value());
 	rec.putField(MYXCHG, inpMyXchg_log->value());
+	rec.putField(CNTY, inpCNTY_log->value());
 	rec.putField(IOTA, inpIOTA_log->value());
 	rec.putField(DXCC, inpDXCC_log->value());
 	rec.putField(CONT, inpCONT_log->value());
@@ -729,6 +736,7 @@ void EditRecord( int i )
 	inpSerNoOut_log->value(editQSO->getField(STX));
 	inpXchgIn_log->value(editQSO->getField(XCHG1));
 	inpMyXchg_log->value(editQSO->getField(MYXCHG));
+	inpCNTY_log->value(editQSO->getField(CNTY));
 	inpIOTA_log->value(editQSO->getField(IOTA));
 	inpDXCC_log->value(editQSO->getField(DXCC));
 	inpCONT_log->value(editQSO->getField(CONT));
@@ -779,6 +787,7 @@ void AddRecord ()
 	inpNotes_log->value (inpNotes->value());
 
 	inpTX_pwr_log->value (progdefaults.mytxpower.c_str());
+	inpCNTY_log->value("");
 	inpIOTA_log->value("");
 	inpDXCC_log->value("");
 	inpCONT_log->value("");
