@@ -492,10 +492,12 @@ bool ARQ_SOCKET_Server::start(const char* node, const char* service)
 
 void ARQ_SOCKET_Server::stop(void)
 {
-	if (!inst)
-		return;
-	delete inst;
-	inst = 0;
+// FILEME - uncomment when we have an ARQ_SOCKET_Server than can be
+// interrupted
+//	if (!inst)
+//		return;
+//	delete inst;
+//	inst = 0;
 }
 
 void* ARQ_SOCKET_Server::thread_func(void*)
