@@ -1,7 +1,7 @@
 // ----------------------------------------------------------------------------
 //      debug.cxx
 //
-// Copyright (C) 2008
+// Copyright (C) 2008-2010
 //              Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.
@@ -26,6 +26,8 @@
 #  include "compat.h"
 #endif
 
+#include <sstream>
+
 #include <cstdio>
 #include <cstring>
 #include <cstdarg>
@@ -61,16 +63,19 @@ static Fl_Double_Window* window;
 static FTextView* text;
 
 debug* debug::inst = 0;
-debug::level_e debug::level = debug::WARN_LEVEL;
+debug::level_e debug::level = debug::INFO_LEVEL;
 uint32_t debug::mask = ~0u;
 
-const char* prefix[] = { _("Quiet"), _("Error"), _("Warning"), _("Info"), _("Debug") };
+static const char* prefix[] = {
+	_("Quiet"), _("Error"), _("Warning"), _("Info"), _("Verbose"), _("Debug")
+};
 
 static void slider_cb(Fl_Widget* w, void*);
 static void src_menu_cb(Fl_Widget* w, void*);
 static void popup_message(void*);
 
 Fl_Menu_Item src_menu[] = {
+	{ _("ARQ control"), 0, 0, 0, FL_MENU_TOGGLE | FL_MENU_VALUE },
 	{ _("Audio"), 0, 0, 0, FL_MENU_TOGGLE | FL_MENU_VALUE },
 	{ _("Modem"), 0, 0, 0, FL_MENU_TOGGLE | FL_MENU_VALUE },
 	{ _("Rig control"), 0, 0, 0, FL_MENU_TOGGLE | FL_MENU_VALUE },
@@ -79,11 +84,32 @@ Fl_Menu_Item src_menu[] = {
 	{ _("Other"), 0, 0, 0, FL_MENU_TOGGLE | FL_MENU_VALUE },
 	{ 0 }
 };
+#include <iostream>
+void debug::rotate_log(const char* filename)
+{
+	const int n = 5; // rename existing log files to keep up to 5 old versions
+	ostringstream oldfn, newfn;
+	ostringstream::streampos p;
+
+	oldfn << filename << '.';
+	newfn << filename << '.';
+	p = oldfn.tellp();
+
+	for (int i = n - 1; i > 0; i--) {
+		oldfn.seekp(p);
+		newfn.seekp(p);
+		oldfn << i;
+		newfn << i + 1;
+		rename(oldfn.str().c_str(), newfn.str().c_str());
+	}
+	rename(filename, oldfn.str().c_str());
+}
 
 void debug::start(const char* filename)
 {
 	if (debug::inst)
 		return;
+	rotate_log(filename);
 	inst = new debug(filename);
 
 	window = new Fl_Double_Window(512, 256, _("Event log"));
@@ -242,6 +268,10 @@ static void src_menu_cb(Fl_Widget* w, void*)
 
 static void popup_message(void* msg)
 {
+	if (window->visible()) {
+		delete [] (char*)msg;
+		return;
+	}
 	if (!Fl::first_window() || !Fl::first_window()->visible()) // defer
 		return Fl::add_timeout(0.5, popup_message, msg);
 
