@@ -350,11 +350,24 @@ bool QRZGetXML(string& xmlpage)
 	return res;
 }
 
+void camel_case(string &s)
+{
+	bool first_letter = true;
+	for (size_t n = 0; n < s.length(); n++) {
+		if (s[n] == ' ') first_letter = true;
+		else if (first_letter) {
+			s[n] = toupper(s[n]);
+			first_letter = false;
+		} else s[n] = tolower(s[n]);
+	}
+}
+
 void QRZ_disp_result()
 {
 	ENSURE_THREAD(FLMAIN_TID);
 
 	if (lookup_fname.length() > 0) {
+		camel_case(lookup_fname);
 		string::size_type spacePos = lookup_fname.find(" ");
 		//    if fname is "ABC" then display "ABC"
 		// or if fname is "A BCD" then display "A BCD"
@@ -414,6 +427,7 @@ void QRZ_CD_query()
 
 	if( qCall->FindRecord( srch ) == 1) {
 		lookup_fname = qCall->GetFname();
+		camel_case(lookup_fname);
 		snip = lookup_fname.find(' ');
 		if (snip != string::npos)
 			lookup_fname.erase(snip, lookup_fname.length() - snip);
@@ -551,9 +565,82 @@ void QRZquery()
 	}
 }
 
-// ----------------------------------------------------------------------------
+// ---------------------------------------------------------------------
+// HTTP:://callook.info queries
+// ---------------------------------------------------------------------
+
+string node_data(const string &xmlpage, const string nodename)
+{
+	size_t pos1, pos2;
+	pos1 = xmlpage.find(string("<").append(nodename).append(">"));
+	if (pos1 == string::npos) return "";
+	pos2 = xmlpage.find(string("</").append(nodename).append(">"));
+	if (pos2 == string::npos) return "";
+	pos1 += (string("<").append(nodename).append(">")).length();
+	return xmlpage.substr(pos1, pos2 - pos1);
+}
+
+void parse_callook(string& xmlpage)
+{
+	string nodestr;
+	nodestr = node_data(xmlpage, "current");
+	if (nodestr.empty()) {
+		lookup_notes = "no data from callook.info";
+		return;
+	}
+	xmlpage = xmlpage.substr(xmlpage.find("</trustee>"));
+	lookup_fname = node_data(xmlpage, "name");
+	camel_case(lookup_fname);
+	nodestr = node_data(xmlpage, "address");
+	if (!nodestr.empty()) {
+		lookup_addr1 = node_data(nodestr, "line1");
+		lookup_addr2 = node_data(nodestr, "line2");
+	}
+	nodestr = node_data(xmlpage, "location");
+	if (!nodestr.empty()) {
+		lookup_lond = node_data(nodestr, "longitude");
+		lookup_latd = node_data(nodestr, "latitude");
+		lookup_grid = node_data(nodestr, "gridsquare");
+	}
+	size_t p;
+	p = lookup_addr2.find(",");
+	if (p != string::npos) {
+		lookup_qth = lookup_addr2.substr(0, p);
+		lookup_addr2.erase(0, p+2);
+		p = lookup_addr2.find(" ");
+		if (p != string::npos)
+			lookup_state = lookup_addr2.substr(0, p);
+	}
+}
+
+bool CALLOOKGetXML(string& xmlpage)
+{
+	string url = string("http://callook.info/").append(callsign).append("/xml");
+	bool res = fetch_http(url, xmlpage, 5.0);
+	LOG_DEBUG("result = %d", res);
+	return res;
+}
+
+void CALLOOKquery()
+{
+	ENSURE_THREAD(QRZ_TID);
+
+	bool ok = true;
+
+	string CALLOOKpage;
+
+	clear_Lookup();
+	ok = CALLOOKGetXML(CALLOOKpage);
+	if (!ok) // change to negative for MS not getting on first try
+		ok = CALLOOKGetXML(CALLOOKpage);
+	if (ok)
+		parse_callook(CALLOOKpage);
+	REQ(QRZ_disp_result);
+}
+
+// ---------------------------------------------------------------------
 // Hamcall specific functions
-// ----------------------------------------------------------------------------
+// ---------------------------------------------------------------------
 
 #define HAMCALL_CALL 181
 #define HAMCALL_FIRST 184
@@ -572,6 +659,7 @@ void parse_html(const string& htmlpage)
 		p++;
 		while ((uchar)htmlpage[p] < 128 && p < htmlpage.length() )
 			lookup_fname += htmlpage[p++];
+			camel_case(lookup_fname);
 	}
 	if ((p = htmlpage.find(HAMCALL_CITY)) != string::npos) { 
 		p++;
@@ -671,6 +759,9 @@ static void *LOOKUP_loop(void *args)
 		case HAMCALLHTML :
 			HAMCALL_DETAILS_query();
 			break;
+		case CALLOOK:
+			CALLOOKquery();
+			break;
 		case QRZ_EXIT:
 			return NULL;
 		default:
@@ -721,6 +812,9 @@ void CALLSIGNquery()
 			return;
 		}
 		break;
+	case CALLOOK:
+		inpNotes->value("Request sent to\nhttp://callook.info...");
+		break;
 	default:
 		LOG_ERROR("Bad query type %d", DB_query);
 		return;
